

//#include <nds.h>
#include "../../common/nds_compati.h"

#include <nds/bios.h>
#include <nds/arm7/touch.h>
#include <nds/arm7/clock.h>


#include "a7sleep.h"



//---------------------------------------------------------------------------------
void startSound(int sampleRate, const void* data, uint32 bytes, u8 channel, u8 vol,  u8 pan, u8 format) {
//---------------------------------------------------------------------------------
	SCHANNEL_TIMER(channel)  = SOUND_FREQ(sampleRate);
	SCHANNEL_SOURCE(channel) = (uint32)data;
	SCHANNEL_LENGTH(channel) = bytes;
	SCHANNEL_CR(channel)     = SOUND_ENABLE | SOUND_ONE_SHOT | SOUND_VOL(vol) | SOUND_PAN(pan) | (format==1?SOUND_8BIT:SOUND_16BIT);
}


//---------------------------------------------------------------------------------
s8 getFreeSoundChannel() {
//---------------------------------------------------------------------------------
	int i;
	for (i=0; i<16; i++) {
		if ( (SCHANNEL_CR(i) & SOUND_ENABLE) == 0 ) return i;
	}
	return -1;
}

void InterruptHandler(void) {
  int t1, t2;
  int i;
  static int heartbeat = 0;
  static int ct=0;


//**********************************************************************
// Put this code into your ARM7 interrupt handler
	if (*((vu32*)0x027FFE24) == (u32)0x027FFE04)  // Check for ARM9 reset
	{
		IME = 0;	// Disable interrupts
		IF = IF;	// Acknowledge interrupt
		{
			a7lcdbl(3);		// LCD ON ON
		}
		*((vu32*)0x027FFE34) = (u32)0x08000000;	// Bootloader start address
		//asm ("swi 0x00\n");	// Jump to boot loader
		swiSoftReset();
	}
//***********************************************************************


  
  if (IF & IRQ_VBLANK) {
    
    // Update the heartbeat
    heartbeat++;
    IPC->heartbeat = heartbeat;

    // Read the X/Y buttons and the /PENIRQ line
//    IPC->buttons = XKEYS;

    u32 xkeys=XKEYS;
    IPC->buttons = xkeys;


    //	Sleep Check
    if(xkeys == 0x00FF)	//	plN[Y
    {
   		a7led(1);

		ct++;
		if(ct<60*10)	// 10bȓȂLCD OFF
		{
			a7lcdbl(0);
		}
		else			//	10b߂dOFF
		{
		//	a7sleep();
		//	ct=0;
			a7poff();
		}
    }
    else
    {
   		a7led(0);
		a7lcdbl(2);
		ct=0;
    }



    // Read the touch screen
    IPC->touchX = touchRead(TSC_MEASURE_X);
    IPC->touchY = touchRead(TSC_MEASURE_Y);
    IPC->touchZ1 = touchRead(TSC_MEASURE_Z1);
    IPC->touchZ2 = touchRead(TSC_MEASURE_Z2);

    // Read the time
    //	AM/PM tȌƂ
    {
    	int i;
    	u8 rtcwk[8];
    	
	    rtcGetTime((uint8 *)rtcwk);
//	    BCDToInteger((uint8 *)&(IPC->curtime[1]), 7);
	    rtcwk[5]&=0x3f;
	    BCDToInteger((uint8 *)&(rtcwk[1]), 7);
	    for(i=1;i<8;i++)
	    {
	    	IPC->curtime[i]=rtcwk[i];
	    }
    }
 
    // Read the temperature
    IPC->temperature = touchReadTemperature(&t1, &t2);
    IPC->tdiode1 = t1;
    IPC->tdiode2 = t2;
    

		//sound code  :)
		TransferSound *snd = IPC->soundData;
		IPC->soundData = 0;

		if (snd) {
			for (i=0; i<snd->count; i++) {
				s8 chan = getFreeSoundChannel();

				if (chan >= 0) {
					startSound(snd->data[i].rate, snd->data[i].data, snd->data[i].len, chan, snd->data[i].vol, snd->data[i].pan, snd->data[i].format);
				}
			}
		}


  }

  // Acknowledge interrupts
  IF = IF;
}


//---------------------------------------------------------------------------------
int main(int argc, char ** argv) {
//---------------------------------------------------------------------------------

	// Reset the clock if needed
	rtcReset();

	//enable sound
	powerON(POWER_SOUND);
	SOUND_CR = SOUND_ENABLE | SOUND_VOL(0x7F);
	IPC->soundData = 0;
 
	// Set up the interrupt handler
	IME = 0;
	IRQ_HANDLER = &InterruptHandler;
	IE = IRQ_VBLANK;
	IF = ~0;
	DISP_SR = DISP_VBLANK_IRQ;
	IME = 1;
	

	// Keep the ARM7 out of main RAM
	while (1) 
	{
		swiWaitForVBlank();
	}
	return 0;
}

 
